import numpy as np
import scipy.signal
import keras.backend as K
from keras.losses import mean_squared_error

from custom_layers import compute_forward_stft, two_sided_to_one_sided, magnitude

log_of_10 = np.log(10.).astype(np.float32)
def K_log10(x):
    return 10.*K.log(x + K.epsilon())/log_of_10


def negative_mse(y_true, y_pred):
    return -mean_squared_error(y_true, y_pred)


def norm_squared(x):
    return K.sum(K.sum(K.square(x), axis=-1), axis=-1)


def dot_product(y_true, y_pred):
    return K.sum(K.sum(y_true*y_pred, axis=-1), axis=-1)


def scale(y_true, y_pred):
    return dot_product(y_true, y_pred) / (K.epsilon() + norm_squared(y_true))


def negative_si_mse(y_true, y_pred):
    return -mean_squared_error(scale(y_true, y_pred)*y_true, y_pred)


def snr(y_true, y_pred):
    return K_log10(K.sum(K.sum(K.square(y_true), axis=-1), axis=-1)) - K_log10(K.sum(K.sum(K.square(y_true - y_pred), axis=-1), axis=-1))


def si_snr(y_true, y_pred):
    return snr(scale(y_true, y_pred)*y_true, y_pred)


def compute_mag_stft(x):
    K_stft = compute_forward_stft(x)
    K_stft1 = two_sided_to_one_sided(K_stft)
    return magnitude(K_stft1)


def negative_mag_mse(y_true, y_pred):
    y_true_mag = compute_mag_stft(y_true)
    y_pred_mag = compute_mag_stft(y_pred)
    return negative_mse(y_true_mag, y_pred_mag)


def mag_snr(y_true, y_pred):
    y_true_mag = compute_mag_stft(y_true)
    y_pred_mag = compute_mag_stft(y_pred)
    return snr(y_true_mag, y_pred_mag)


def negative_si_mag_mse(y_true, y_pred):
    return negative_mag_mse(scale(y_true, y_pred)*y_true, y_pred)


def si_mag_snr(y_true, y_pred):
    return mag_snr(scale(y_true, y_pred)*y_true, y_pred)
