import numpy as np
np.random.seed(1234)
import scipy.signal
import librosa

import tensorflow as tf
from keras.backend.tensorflow_backend import set_session
config = tf.ConfigProto()
config.gpu_options.allow_growth = True  # dynamically grow the memory used on the GPU
sess = tf.Session(config=config)
set_session(sess)  # set this TensorFlow session as the default session for Keras
from keras.models import Model
from keras.layers import Input

from custom_layers import build_forward_stft, TwoSidedToOneSided

if __name__=="__main__":

    # define parameters
    batch_size = 1
    samples = 1664
    window_len = 512
    window=np.sqrt(scipy.signal.hann(window_len, sym=False).astype(np.float32))
    hop_len = window_len / 4
    num_frames = 1 + (samples - window_len) / hop_len

    # build the keras stft
    x_input = Input(shape=(samples, 1))
    K_stft, K_stft_layers = build_forward_stft(x_input, window, hop_len, Nfft=None, trainable_params=[], dense_window=False, unitary_dft=False, layers_spec={}, dft_scale='sqrtn', complex_input=False, input_type='raw')
    model_stft = Model(x_input, K_stft)
    K_stft1 = TwoSidedToOneSided(K_stft)
    model_stft1 = Model(x_input, K_stft1)

    # sample a signal
    x = np.random.randn(batch_size, samples, 1)

    # compute keras stft
    X_keras_realimag = model_stft.predict(x) # real stacked on top of imaginary
    # convert from composite real/imag to complex
    X_keras = X_keras_realimag[0, :, :window_len].T + 1j*X_keras_realimag[0, :, window_len:].T
    # remove redundant frequencies (convert to one-sided)
    X_keras = X_keras[:window_len/2+1, :]

    # compute one-sided keras stft
    X1_keras_realimag = model_stft1.predict(x) # real stacked on top of imaginary
    # convert from composite real/imag to complex
    freqs = X1_keras_realimag.shape[-1]/2
    X1_keras = X1_keras_realimag[0, :, :freqs].T + 1j*X1_keras_realimag[0, :, freqs:].T

    # compute librosa stft
    X_librosa = librosa.stft(np.squeeze(x), n_fft=window_len, hop_length=hop_len, win_length=window_len, window=window, center=False)
    X_librosa = np.conj(X_librosa)/np.sqrt(np.sum(window_len))

    mse = np.mean( np.abs(X_librosa - X_keras)**2 )
    print "MSE between keras and librosa STFTs is %e" % mse
    mse1 = np.mean( np.abs(X_librosa - X1_keras)**2 )
    print "MSE between one-sided keras and librosa STFTs is %e" % mse1

