from functools import partial
import numpy as np
import scipy
import tensorflow as tf
from tensorflow.contrib.signal.python.ops import window_ops
import librosa
import keras

from custom_layers import STFTLayer, ISTFTLayer

def sqrt_hann(frame_length, dtype=None):
  return tf.sqrt(window_ops.hann_window(frame_length, periodic=True))

def inv_sqrt_hann(frame_length, frame_step=None, dtype=None):
  if frame_step is None:
      frame_step = frame_length // 4
  return tf.contrib.signal.inverse_stft_window_fn(frame_step, forward_window_fn=sqrt_hann)

if __name__=="__main__":
    wav_ref = 'chime2-wsj0/scaled/si_et_05/0dB/wsj0/si_et_05/440/440c0201.wav'

    frame_length = 512
    frame_step = frame_length // 4

    sess = tf.Session()

    x_ref, fs = librosa.load(wav_ref, sr=16000)
    samples = len(x_ref)
    x = np.reshape(x_ref, (1, samples))
    X_tf = tf.contrib.signal.stft(x, frame_length, frame_step,
                                  window_fn=sqrt_hann,
                                  pad_end=True)

    xr_tf = tf.contrib.signal.inverse_stft(X_tf, frame_length, frame_step,
                                           window_fn=tf.contrib.signal.inverse_stft_window_fn(frame_step, forward_window_fn=sqrt_hann))

    X = X_tf.eval(session=sess)
    xr = xr_tf.eval(session=sess)[0, :len(x_ref)]
    scale = np.mean(x_ref[frame_length:-frame_length]/xr[frame_length:-frame_length])
    print "scale is %f" % scale
    mse = np.mean((x_ref - xr)**2)
    nmse = mse / np.mean(x_ref**2)
    print "MSE of reconstruction is %e" % mse
    print "NMSE of reconstruction is %e" % nmse

    model = keras.models.Sequential()
    model.add(STFTLayer(frame_length, frame_step, input_shape=(samples,)))
    istft = ISTFTLayer(frame_length, frame_step, samples=len(x_ref))(model.output)
    model2 = keras.models.Model(model.input, istft)
    print "Done."
